<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: SCMRepository.php 4537 2010-08-24 13:11:27Z tperdue $
 * @author Marcelo Mottalli	marcelo@gforgegroup.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('SystemUtils.php');
require_once('plugins/scm/lib/ProjectScmPeer.php');
require_once('plugins/scm/lib/ScmPreference.php');
require_once('plugins/scm/lib/ScmModule.php');
require_once('plugins/scm/lib/ScmPlugin.php');
require_once('plugins/scm/lib/ScmModuleUser.php');
require_once('core/UsergroupUser.php');
		

abstract class SCMRepository {
	protected $project;
	protected $repositoryName;
	
	public function __construct(Project $project) {
		$this->project = $project;
		$this->repositoryName = $project->getUnixName();
	}
	
	public function isConfigured() {
		$config = $this->getScmConfiguration();
		return (!is_null($config));
	}
	
	public function getRepositoryName() {
		return $this->repositoryName;
	}
	
	public function getProject() {
		return $this->project;
	}
	
	/**
	 * Returns the ProjectScm object for this project
	 * @return	ProjectScm
	 */
	public function getScmConfiguration() {
		$pscm = ProjectScmPeer::retrieveByPk($this->project->getProjectId());
		return $pscm;
	}
	
	/**
	 * Returns the ScmPreference object for this project
	 * @return	ScmPreference
	 */
	public function getScmPreferences() {
		$pscm = ScmPreferencePeer::retrieveByPk($this->project->getProjectId());
		return $pscm;
	}

	/**
	 * Get the stats from the repository
	 * @return array
	 */
	public function getStats() {
		// Get the SCM stats
		$sql = 'SELECT SUM(checkouts) AS total_checkouts, SUM(commits) AS total_commits, SUM(adds) AS total_adds
				FROM scm_project_stats
				WHERE project_id='.$this->project->getProjectId();
		$con = Propel::getConnection(ScmProjectStatsPeer::DATABASE_NAME);
		$rs = $con->createStatement()->executeQuery($sql);
		$rs->next();
		
		$checkouts = $rs->getInt('total_checkouts');
		$adds = $rs->getInt('total_adds');
		$commits = $rs->getInt('total_commits');
		
		return array(
			'commits'	=> $commits,
			'adds'		=> $adds,
			'checkouts'	=> $checkouts
		);
	}

	/**
	 * Return the type of this repository (ie, 'cvs', 'svn', etc)
	 * @return	string
	 */
	public abstract function getType();
	
	/**
	 * Returns the for this repository
	 * @return	string
	 */
	public abstract function getRoot();
	
	/**
	 * Returns the root for ALL repositories
	 * @return	string
	 */
	public abstract function getScmRoot();
	
	public function getModules() {
		$c = new Criteria();
		$c->add(ScmModulePeer::PROJECT_ID, $this->project->getProjectId());
		return ScmModulePeer::doSelect($c);
	}
	
	/**
	 * Get the access control list for the module
	 */
	public function getModulePermissions($scm_module_id) {
		$sql = '
				SELECT u.*, sq.value, sq.access 
				FROM (
					SELECT u.user_id, MAX(rs.value) AS value, smu.access
					FROM "user" as u, role_setting AS rs, user_project_role AS upr, user_project AS up
						LEFT JOIN scm_module AS sm ON sm.project_id=up.project_id
						LEFT JOIN scm_module_user AS smu ON smu.scm_module_id=sm.scm_module_id 
							AND smu.user_id=up.user_id
					WHERE up.user_id=u.user_id AND upr.user_project_id=up.user_project_id AND rs.role_id=upr.role_id 
					AND up.project_id=? AND rs.section=? AND rs.value >= ? AND sm.scm_module_id=?
					GROUP BY u.user_id, smu.access
				) AS sq, "user" as u
				WHERE u.user_id=sq.user_id;
				';
		$con = Propel::getConnection(ScmModulePeer::DATABASE_NAME);
		$stmt = $con->prepareStatement($sql);
		$stmt->setInt(1, $this->project->getProjectId());
		$stmt->setString(2, 'scm');
		$stmt->setInt(3, ScmPlugin::READ);
		$stmt->setInt(4, $scm_module_id);

		$ret = array();
		$rs = $stmt->executeQuery(ResultSet::FETCHMODE_NUM);
		while ($rs->next()) {
			$user = new User();
			$user->hydrate($rs);
			
			$j = UserPeer::$numColumns - UserPeer::$numLazyLoadColumns + 1;
			$scm_access = $rs->getInt($j);
			$module_access = $rs->getInt($j+1);
			
			$ret[] = array("user" => $user, 
							"scm_access" => $scm_access, 
							"module_access" => $module_access);
		}
		unset($rs);
		
		return $ret;
	}
	
	/**
	 * Get the access permissions for a user in a certain module
	 */	
	public function getModulePermissionsForUser($scm_module_id, $user_id) {
		// The user may have group or individual permissions for the module. Individual permissions take
		// precedence
	
		$c = new Criteria();
		$c->add(ScmModuleUserPeer::SCM_MODULE_ID, $scm_module_id);
		$c->add(ScmModuleUserPeer::USER_ID, $user_id);
		$module_user = ScmModuleUserPeer::doSelectOne($c);
		
		if (!is_null($module_user)) {
			return $module_user->getAccess();
		}
		
		// Permissions not set for the user in this module. We must check if the user belongs to a 
		// user group and check the group's permissions
		$c = new Criteria();
		$c->add(ScmModuleUserPeer::SCM_MODULE_ID, $scm_module_id);
		$c->addJoin(ScmModuleUserPeer::USER_ID, UsergroupUserPeer::USERGROUP_ID);
		$c->add(UsergroupUserPeer::MEMBER_USER_ID, $user_id);
		$module_groups = ScmModuleUserPeer::doSelect($c);
		
		// Get the most permissive permission
		$access = 0;
		foreach ($module_groups as $module_group) {
			$access = max($access, $module_group->getAccess());
		}
		
		if(!$access){
			return null;
		} else {
			return $access;
		}
	}
	
	public function userIsAdmin() {
		return $this->project->userIsAdmin();
	}
	
	/**
	 * Checks if the current user can commit to $path
	 */
	public function userCanCommit($path) {
		// Project admin can always commit
		if ($this->project->userIsAdmin()) {
			return true;
		}
		
		// Check if $path corresponds to some module
		$modules = $this->getModules();
		$match_module = null;
		foreach ($modules as $module) {
			if ($this->isPartOfModule($path, $module->getPath())) {
				// The following check allows us to match $path with the innermost module
				if (is_null($match_module) || $this->isPartOfModule($module->getPath(), $match_module->getPath())) {
					$match_module = $module;
				}
			}
		}

		$perm = $this->project->getPermission();
		// See if the user has permissions on the repository
		$scm_can_write = $perm->check(ScmPlugin::SECTION, 0, ScmPlugin::WRITE);
		if (is_null($match_module)) {
			// Permissions for this $path aren't specified... just see if the user
			// has permissions for the general repository
			return $scm_can_write;
		}
		
		// Here we must check if the user has permissions in the module
		$user = Session::getSession()->getUser();
		
		$access = $this->getModulePermissionsForUser($match_module->getScmModuleId(), $user->getUserId());
		if (is_null($access)) {
			// Permissions not set for this user in this module
			return $scm_can_write;
		} else {
			return ($access >= ScmPlugin::WRITE);
		}
	}
	
	/**
	 * Checks if the current user has access to $path
	 */
	public function userHasAccess($path) {
		// If the repository is public, we always have at least read-only access
		$project_scm = ProjectScmPeer::retrieveByPk($this->project->getProjectId());
		if ($this->project->getIsPublic() && $project_scm && $project_scm->getEnableAnonread()) {
			return true;
		}
		
		// Project admin always has access
		if ($this->project->userIsAdmin()) {
			return true;
		}
		
		// Check if $path corresponds to some module
		$modules = $this->getModules();
		$match_module = null;
		foreach ($modules as $module) {
			if ($this->isPartOfModule($path, $module->getPath())) {
				// The following check allows us to match $path with the innermost module
				if (is_null($match_module) || $this->isPartOfModule($module->getPath(), $match_module->getPath())) {
					$match_module = $module;
				}
			}
		}

		$perm = $this->project->getPermission();
		// See if the user has permissions on the repository
		$scm_has_access = $perm->check(ScmPlugin::SECTION, 0, ScmPlugin::READ);		// At least read access
		if (is_null($match_module)) {
			// Permissions for this $path aren't specified... just see if the user
			// has permissions for the general repository
			return $scm_has_access;
		}
		
		// Here we must check if the user has permissions in the module
		$user = Session::getSession()->getUser();
		
		$access = $this->getModulePermissionsForUser($match_module->getScmModuleId(), $user->getUserId());
		if (is_null($access)) {
			// Permissions not set for this user in this module
			return $scm_has_access;
		} else {
			return ($access >= ScmPlugin::READ);
		}
	}

	/**
	 * Checks if $dir is equal to or is inside $module. Both parameters must be 
	 * _absolute_ paths.
	 * @param	string $dir
	 * @param	string $module 
	 */
	private function isPartOfModule($dir, $module) {
		// Remove trailing slashes, and replace double-slashes (just in case)
		if ($module[strlen($module)-1] != '/') {
			$module .= '/';
		}
		if ($dir[strlen($dir)-1] != '/') {
			$dir .= '/';
		}
		//$dir = preg_replace('/\\/+$/', '', $dir);
		//$module = preg_replace('/\\/+$/', '', $module);
		$dir = preg_replace('/\\/+/', '/', $dir);
		$module = preg_replace('/\\/+/', '/', $module);
		
		// make sure both start with a slash
		if (strpos($dir, '/') !== 0) $dir = '/'.$dir;
		if (strpos($module, '/') !== 0) $module = '/'.$module;
		
		// $dir is inside $module if and only if it starts with $module
		return (strpos($dir, $module) === 0);
	}
	
	/**
	 * Performs a local checkout of the specified module/directory into $destdir (must have write permissions
	 * by the current user)
	 */	
	public /*abstract*/ function doLocalCheckout($destdir, $module) {
		// Must implmement this for each child class
		throw new Exception('doLocalCheckout() not implemented for '.$this->getType());
	}
	
	public /*abstract*/ function getCheckedoutFileRevision($checkout_dir, $file) {
		// Must implmement this for each child class
		return -1;			// Default value
	}
}
?>
