<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


require_once('plugins/scm/lib/SCMRepository.php');
require_once('plugins/scmperforce/lib/PerforceManager.php');
require_once('plugins/scmperforce/lib/PerforceConfigFile.php');

class PerforceRepository extends SCMRepository {
	private $perforceManager;
	
	public function __construct(Project $project) {
		parent::__construct($project);
		
		$this->perforceManager = new PerforceManager();
	}
	
	public function getType() {
		return 'perforce';
	}
	
	public function getRoot() {
		return '//'.$this->repositoryName.'/';
	}
	
	public function getScmRoot() {
		return '//'.$this->repositoryName.'/';
	}
	
	public function isCreated() {
		$depots = $this->perforceManager->getDepots();
		
		return array_key_exists($this->repositoryName, $depots);
	}
	
	/**
	 * Creates a new repository ("depot") for this project
	 */
	public function create() {
		$confName = tempnam('/tmp', PerforceManager::CONFIG_PREFIX);
		$confFile = new PerforceConfigFile($confName);
		
		$confFile->setVar('Depot', $this->repositoryName);
		$confFile->setVar('Description', 'Depot for GForge project "'.$this->project->getName().'"');
		$confFile->setVar('Type', 'local');
		$confFile->setVar('Map', $this->repositoryName.'/...');
		$confFile->save();
		
		$manager = new PerforceManager();
		$manager->setDepot($confName);
		unlink($confName);
		
		$this->updateRepository();
	}
	
	public function updateRepository() {
		$manager = new PerforceManager();
		
		$this->updateGroups();
		
		// Commands to execute
		$preCommitCmd = '"php '.SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/scmperforce/bin/pre_commit.php').' '.$this->repositoryName.' %changelist% %user%"';
		$postCommitCmd = '"php '.SystemUtils::buildFilePath(GFConf::get('system.pluginsPath'), '/scmperforce/bin/post_commit.php').' '.$this->repositoryName.' %changelist% %user%"';

		// Add the commit triggers
		$manager->setTrigger($this->repositoryName.'_pre_commit', 'change-submit', '//'.$this->repositoryName.'/...', $preCommitCmd);
		$manager->setTrigger($this->repositoryName.'_post_commit', 'change-commit', '//'.$this->repositoryName.'/...', $postCommitCmd);
	}
	
	/**
	 * Creates/updates the Perforce groups associated to this project. There are two groups: a
	 * read-only group and a group with commit permissions.
	 */
	private function updateGroups() {
		$manager = new PerforceManager();
		
		$readers = array();
		$committers = array();
		
		// Get the list of users for this group
		$users = $this->project->getMembers();
		foreach ($users as $user) {
			$permission = $this->project->getUserPermission($user);
			if ($permission->check(ScmPlugin::SECTION, 0, ScmPlugin::WRITE)) {
				$committers[] = $user->getUnixName();
			} else if ($permission->check(ScmPlugin::SECTION, 0, ScmPlugin::READ)) {
				$readers[] = $user->getUnixName();
			} else {
				// User doesn't have access to the repository
			}
		}
		
		// See if other users have read access
		$project_scm = ProjectScmPeer::retrieveByPk($this->project->getProjectId());
		if (!is_null($project_scm) && $project_scm->getEnableAnonread() && $this->project->getIsPublic()) {
			$readers[] = 'anonymous';
		}

		// Create/update the readers' group
		$readersGroupName = $this->project->getUnixName().'_readers';
		$confName = tempnam('/tmp', PerforceManager::CONFIG_PREFIX);
		$confFile = new PerforceConfigFile($confName);
		$confFile->setVar('Group', $readersGroupName);
		$confFile->setVar('Users', $readers);
		$confFile->save();
		$manager->setGroup($confName);
		unlink($confName);

		// Create/update the committers' group
		$committersGroupName = $this->project->getUnixName().'_committers';
		$confName = tempnam('/tmp', PerforceManager::CONFIG_PREFIX);
		$confFile = new PerforceConfigFile($confName);
		$confFile->setVar('Group', $committersGroupName);
		$confFile->setVar('Users', $committers);
		$confFile->save();
		$manager->setGroup($confName);
		unlink($confName);
		
	
		// Update the permissions for both groups
		$manager->setProtect(PerforceManager::PROTECT_READ, 'group', $readersGroupName, '*', '//'.$this->repositoryName.'/...');
		$manager->setProtect(PerforceManager::PROTECT_WRITE, 'group', $committersGroupName, '*', '//'.$this->repositoryName.'/...');
	}

}
?>